<h1 align="left">
    <img src="https://github.com/helightdev/dogs/blob/main/docs/assets/logo.png?raw=true" align="left" width="48" alt="Frosty">
    Dart Object Graphs
    <a href="">
        <img src="https://img.shields.io/pub/v/dogs_core" alt="pub">
    </a>
    <a href="https://discord.gg/6HKuGSzYKJ">
        <img src="https://img.shields.io/github/actions/workflow/status/helightdev/dogs/dart.yaml" alt="Build">
    </a>
    <a href="https://dogs.helight.dev/">
        <img src="https://img.shields.io/badge/docs-dogs.helight.dev-007ec6.svg" alt="gitbook">
    </a>
</h1>

DOGs, short for Dart Object Graphs, is a object mapping and serialization library that use
code generation to create a structure definition of your serializable classes, which is usable at
runtime.

```dart
@serializable
class Person with Dataclass<Person>{

  @LengthRange(max: 128)
  final String name;

  @Minimum(18)
  final int age;

  @SizeRange(max: 16)
  @Regex("((_)?[a-z]+[A-Za-z0-9]*)+")
  final Set<String>? tags;

  Person(this.name, this.age, this.tags);

}
```

* 🐦 **Concise** Write expressive code with effectively zero boilerplate
* 🚀 **Fast** Similar or increased performance compared to alternatives
* 🧩 **Extensible** Modify and customize any part of your serialization
* 📦 **Adaptive** Use one of the various formats or bring your own
* 📚 **Documented** Comprehensive documentation and examples
* ⚡ **Unmatched DX** No part files, no dollar signs, no getters, no hassle

## Format Support
- **JSON** (included in dogs_core)
- **YAML** <a href="https://pub.dev/packages/dogs_yaml"><img src="https://img.shields.io/pub/v/dogs_yaml?label=dogs_yaml" alt="pub"></a>
- **TOML** <a href="https://pub.dev/packages/dogs_toml"><img src="https://img.shields.io/pub/v/dogs_toml?label=dogs_toml" alt="pub"></a>
- **CBOR** <a href="https://pub.dev/packages/dogs_cbor"><img src="https://img.shields.io/pub/v/dogs_cbor?label=dogs_cbor" alt="pub"></a>

## Core Package <a href="https://pub.dev/packages/dogs_core"><img src="https://img.shields.io/pub/v/dogs_core?label=dogs_core" alt="pub"></a>
- **Json**  
  The core package comes with json support out of the box and requires no additional dependencies.


- **Builders**  
  We provide autogenerated builders, similar to built_value, which allow you to easily create
  immutable objects with a fluent api.


- **Dataclasses**  
  Using our dataclass mixin automatically implements toString, equals and hashCode for your
  serializable classes. This removes the requirement for a package like equatable or
  ungodly amounts of boilerplate code.


- **Projections**  
  Dogs offers a powerful projection api, which allows you to transform your objects into
  other objects, without having to write additional boilerplate code.


- **Polymorphism**  
  Dogs supports polymorphic serialization, which allows you to serialize and deserialize
  objects of different types. You can even use interfaces and abstract classes as types,
  including `Object`.


- **Validation**  
  Using the validation api, you can easily validate your objects, without having the struggle
  of writing your own validation logic for most common use cases.


- **OpenApi Schema** (preview)  
  Dogs offers a schema generation api, which allows you to generate an OpenApi schema for your
  objects, to be used with OpenApi tools like Swagger.

[Package on pub.dev](https://pub.dev/packages/dogs_core)  
[Documentation](https://dogs.helight.dev)

## Firestore <a href="https://pub.dev/packages/dogs_firestore"><img src="https://img.shields.io/pub/v/dogs_firestore?label=dogs_firestore" alt="pub"></a>
Dogs offers a firestore api, which allows you to easily create firestore documents from your
serializable objects and vice versa. You can either use our lightweight extensions on the Firestore
classes or make use of our entity api, which offers you a simplified and more object oriented way of
interacting with firestore.  
[Package on pub.dev](https://pub.dev/packages/dogs_firestore)  
[Documentation](https://dogs.helight.dev/firestore/)

## Forms <a href="https://pub.dev/packages/dogs_forms"><img src="https://img.shields.io/pub/v/dogs_forms?label=dogs_forms" alt="pub"></a>
Use our forms api to easily create forms for any serializable object, without having
to write boilerplate code. The package uses flutter_form_builder under the hood, so you can
benefit from its large ecosystem of form fields.  
[Package on pub.dev](https://pub.dev/packages/dogs_forms)  
[Documentation](https://dogs.helight.dev/forms/)

## ✨ Fancy Code Generation ✨
Dogs doesn't use part files for generated code but instead
generated files for all serializable classes, which then get exported in the
dogs.g.dart file at the root of your lib folder. This massively improves the
developer experience, as you **don't** have to write **annoying part statements** with
missing files everytime. In fact, you don't need do **import anything generated** in
your **model files** at all! Generally, the api of dogs is designed to be independent
of generated code, where it makes sense. All serialization and deserialization
methods don't require any reference to generated code and can be used without
any imports. The only exception to this are the builders, which are generated
for each serializable class and require the dogs.g.dart file to be imported.
